@@ -1,6 +1,6 @@
 package Crypt::ECDSA;
 
-our $VERSION = '0.047';
+our $VERSION = '0.05';
 
 use strict;
 use warnings;
@@ -13,11 +13,12 @@ use Crypt::ECDSA::Util qw( bint random_bits );
 
 require Exporter;
 require DynaLoader;
-
 our @ISA = qw(Exporter DynaLoader);
 our @EXPORT = qw( multiply_F2m invert_F2m );
 bootstrap Crypt::ECDSA $VERSION;
 
+my $DEBUG = 0;
+
 sub new {
     my ( $class, %args ) = @_;
     my $self = {};
@@ -55,15 +56,10 @@ sub make_text_digest {
     return Rmpz_init_set_str( $self->{algo}->hexdigest, 16 );
 }
 
-sub _make_k_kinv_pair {
+sub _make_ephemeral_k {
     my( $self, $n ) = @_;
-    my $len = length( Rmpz_get_str( $n, 16 ) );
-    my $k = Rmpz_init();
-    my $kinv = Rmpz_init();
-    $k = random_bits( ($len + 8) * 16 ) % ( $n - 1 ) + 1;
-    Rmpz_invert( $kinv, $k, $n );
-    return unless Rmpz_sgn($k) and Rmpz_sgn($kinv);
-    return ($k, $kinv);
+    my $len = ( length( Rmpz_get_str( $n, 16 ) ) + 8 ) * 16;
+    return random_bits($len) % ( $n - 1 ) + 1;
 }
 
 sub signature {
@@ -82,24 +78,38 @@ sub signature {
         carp("Cannot sign curve without a point of known order");
         return;
     }
-    my $e = $args{hash} 
-      || ( $args{message} ? $self->make_text_digest($args{message}) : 
+    if( $args{ message_file} ) {
+        open( my $infh, '<', $args{message_file} )
+          or croak( "cannot open file $args{message_file}: $!" );
+        binmode $infh;
+        read( $infh, $args{message}, -s $infh );
+        close $infh;
+    }
+    my $e = $args{hash}
+      || ( $args{message} ? $self->make_text_digest($args{message}) :
            croak("Need a message or the hash of a message for the signature" ) )
     ;
     my $d = bint( $self->{key}->{d} );
     my $max_tries = 12;
     my($r, $s);
     while ( $max_tries-- ) {
-        my($k, $kinv) = $self->_make_k_kinv_pair($q);
+        my $k = $self->_make_ephemeral_k($q);
         my $kG = $G * $k;
-        $r  = bint( $kG->{X} );
-        next unless Rmpz_sgn($r);
-        $s = ( $kinv * ( $e + ( $d * $r )% $n ) ) % $n;
-        next unless Rmpz_sgn($s);
+        $r  = bint( $kG->X % $n ) ;
+        next if $r == 0;
+        my $kinv = Rmpz_init();
+        Rmpz_invert( $kinv, $k, $n );
+        $s = bint( ( $kinv * ( $e + ($d * $r) % $n ) ) % $n );
+        next if $s == 0;
+        if( $args{sig_file} ) {
+            Crypt::ECDSA::PEM::write_ECDSA_signature_file
+              ( $args{sig_file}, bint($r), bint($s) );
+        }
+        warn "signature: r is $r, s is $s\n" if $DEBUG;
         return ( $r, $s );
     }
     $self->{_last_ECDSA_error} =
-      "Failed with getting digest (r,s) = ($r, $s) in signature after $max_tries tries";
+      "Failed with getting digest (r,s) in signature after $max_tries tries";
     return;
 }
 
@@ -116,19 +126,32 @@ sub verify {
     my ( $self, %args ) = @_;
     my $key = $self->key;
     # some error checks
-    my $q = $self->key->curve->q;
-    my $G = $self->key->{G};
-    my $n = $self->key->curve->order;
+    my $q = $key->curve->q;
+    my $G = $key->{G};
+    unless( defined $key->{Q} ) {
+        carp( "Need a Q point in key for ecdsa verify" );
+        return;
+    }
+    my $n = $key->curve->order;
     unless($n) {
         carp("Cannot verify public point without knowing its base point's order");
         return;
     }
-    my $r = $args{r};
-    my $s = $args{'s'};
+    my( $r, $s ) = $args{sig_file}
+      ? Crypt::ECDSA::PEM::read_ECDSA_signature_file( $args{sig_file} )
+      : ( $args{r}, $args{'s'} )
+    ;
     return if $r < 1 or $r >= $q;
     return if $s < 1 or $s >= $q;
-    my $e = $args{hash} 
-      || ( $args{message} ? $self->make_text_digest($args{message}) : 
+    if( $args{ message_file} ) {
+        open( my $infh, '<', $args{message_file} )
+          or croak( "cannot open file $args{message_file}: $!" );
+        binmode $infh;
+        read( $infh, $args{message}, -s $infh );
+        close $infh;
+    }
+    my $e = $args{hash}
+      || ( $args{message} ? $self->make_text_digest($args{message}) :
            croak("Need a message or the hash of a message for the signature" ) )
     ;
     my $w  = Rmpz_init;
@@ -142,7 +165,10 @@ sub verify {
     $u_g = ( $e * $w ) % $n;
     $u_r = ( $r * $w ) % $n;
     my $prod = $G * $u_g + $key->Q * $u_r;
-    return 1 if $prod->{X} % $n == $r;
+    return 1 if $prod->X % $n  == $r;
+
+    warn "verify: r is $r and prod->X mod n is ", $prod->X % $n, "\n" if $DEBUG;
+
     return 0;
 }
 
@@ -156,31 +182,31 @@ Crypt::ECDSA -- Elliptical Cryptography Digital Signature Algorithm
     using the Math::GMPz library and a little C for speed.
 
     Implements the pending FIPS 186-3 ECDSA standard for digital signatures using
-    elliptical key crytography.  Like FIPS 186-3, this is preliminary-- not yet 
-    ready for full use.  It does contain a working implementation of the elliptical 
-    key crypto found in the current 186-2 standard.  The final details of the 186-3 
+    elliptical key crytography.  Like FIPS 186-3, this is preliminary-- not yet
+    ready for full use.  It does contain a working implementation of the elliptical
+    key crypto found in the current 186-2 standard.  The final details of the 186-3
     standard are still being worked out.  Perhaps a preliminary version of signature
     in the NEW standard might be the following, which uses SHA-256 instead of the
     current SHA-1 digest:
-    
-    my $ecdsa = Crypt::ECDSA->new( 
+
+    my $ecdsa = Crypt::ECDSA->new(
       standard => 'ECP-256',
       algorithm => Digest::SHA->new(256);
     );
-    my $msg = "This is a test message for perl ecdsa."    
+    my $msg = "This is a test message for perl ecdsa."
     my ( $r, $s ) = ecdsa->signature( message => $msg );
 
     print "Signature (r, s) is: \nr = $r\ns = $s\n";
-    
-    
+
+
 =head1 SYNOPSIS
 
     my $ecdsa = Crypt::ECDSA->new( standard => 'ECP-256' );
-    
+
     my $msg = "This is a test message for perl ecdsa."
-    
+
     my ( $r, $s ) = ecdsa->signature( message => $msg );
-    
+
     my $verify_ok = $ecdsa->verify( r => $r, 's' => $s, message => $msg );
 
     my $ecdsa_from_PEM = Crypt::ECDSA->new( PEM => $pem_filename );
@@ -191,30 +217,30 @@ Crypt::ECDSA -- Elliptical Cryptography Digital Signature Algorithm
 =over 4
 
 =item B<new>
-  
+
   Create an ECDSA object.
-  
+
   Arguments include:
-  
+
   standard => curve type, one of 'ECP-192', 'ECP-224', 'ECP-256', 'ECP-384',
     'ECP-521', 'EC2N-163', 'EC2N-233', 'EC2N-283', 'EC2N-409', 'EC2N-571',
-    
+
   algorithm => $algo,  where $algo is a Digest::SHA interface compatible object,
     which defaults to Digest::SHA(1) which does SHA-1 digests for ECDSA.
-    
-  .. and other arguments, used as per Crypt::ECDSA::Key.
-  
 
-  
-  
-  
+  .. and other arguments, used as per Crypt::ECDSA::Key.
+
+
+
+
+
 =item B<key>
 
   my $key = $ecdsa->key;
 
   Get the key object in use by this ecdsa object
 
-=item B<errstr> 
+=item B<errstr>
 
   print $ecdsa->errstr;
 
@@ -223,24 +249,24 @@ Crypt::ECDSA -- Elliptical Cryptography Digital Signature Algorithm
 =item B<keygen>
 
   if( $want_new_key ) {  $
-    my( $secret, $base_point ) = ecdsa->keygen(); 
+    my( $secret, $base_point ) = ecdsa->keygen();
 
   Make a new private/ public key pair
 
 =item B<make_text_digest>
 
     my $msg = "This is a test message fpr perl ecdsa."
-    
+
     my  $digest = ecdsa->make_text_digest( $msg );
 
   Make a text digest via the algorithm passed to new ( default is SHA-1 )
 
-  
+
 =item B<signature>
 
-    
+
     my ( $r, $s ) = ecdsa->signature( message => $msg );
-    
+
   Sign a message as message => message or a digest as hash => $digest
 
 =item B<sign>
@@ -249,7 +275,7 @@ Crypt::ECDSA -- Elliptical Cryptography Digital Signature Algorithm
 
 =item B<verify_public_key>
 
-  Verify a public key point, as in the Crypt::ECDSA::Key method 
+  Verify a public key point, as in the Crypt::ECDSA::Key method
 
 =item B<verify>
 
@@ -267,17 +293,17 @@ Crypt::ECDSA -- Elliptical Cryptography Digital Signature Algorithm
 =head1 NOTES
 
 =item B<See FIPS 186-3, draft standard>
-  Note the use of SHA-1 hashing is becoming deprecated, but is still the default.  
+  Note the use of SHA-1 hashing is becoming deprecated, but is still the default.
   SHA-256 hashing may be used instead of SHA-1 when practicable.
 
 =item See also L<http://en.wikipedia.org/wiki/Elliptic_Curve_DSA>, quoted below:
 
   Signature generation algorithm
 
-  Suppose Alice wants to send a signed message to Bob. 
-  Initially, the curve parameters (q,FR,a,b,G,n,h) must be agreed upon. 
-  Also, Alice must have a key pair suitable for elliptic curve cryptography, 
-  consisting of a private key dA (a randomly selected integer in the 
+  Suppose Alice wants to send a signed message to Bob.
+  Initially, the curve parameters (q,FR,a,b,G,n,h) must be agreed upon.
+  Also, Alice must have a key pair suitable for elliptic curve cryptography,
+  consisting of a private key dA (a randomly selected integer in the
   interval [1,n - 1]) and a public key QA (where QA = dAG).
 
   For Alice to sign a message m, she follows these steps:
@@ -290,7 +316,7 @@ Crypt::ECDSA -- Elliptical Cryptography Digital Signature Algorithm
 
   Signature verification algorithm
 
-  For Bob to authenticate Alice's signature, he must have a copy of her 
+  For Bob to authenticate Alice's signature, he must have a copy of her
   public key QA. He follows these steps:
 
    1. Verify that r and s are integers in [1,n - 1]. If not, the signature is invalid.
@@ -301,7 +327,7 @@ Crypt::ECDSA -- Elliptical Cryptography Digital Signature Algorithm
    6. The signature is valid if x1 = r(mod n), invalid otherwise.
 
 =head1  TODO
-   
+
     The Koblitz curve point multiplication algorithm could be optimized a bit more.
     Digital X.509 certificate handling might be implemented for OpenSSL compatibility.
 
@@ -316,7 +342,7 @@ William Herrera (wherrera@skylightview.com)
   This module is free software; you can redistribute it and/or modify it
   under the same terms as Perl itself.
 
- 
+
 =cut
 
 1;
